/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

// undef the GetCurrentTime macro defined in WinBase.h from the MS Platform SDK
%{C++
#ifdef GetCurrentTime
#undef GetCurrentTime
#endif
%}

[scriptable, builtinclass, uuid(3ef760bb-ff19-4dbb-b552-af27ab84b9b8)]
interface nsIDOMMediaStream : nsISupports
{
  /* Placeholder interface only; will be removed after further WebIDL conversion.
     Do not add anything here. */
};

[scriptable, builtinclass, uuid(dd37150a-9823-4605-ac4c-3516629a8aaf)]
interface nsIDOMLocalMediaStream : nsIDOMMediaStream
{
  /* Placeholder interface only; will be removed after further WebIDL conversion.
     Do not add anything here. */
};
